==> bson-4.1.0.redhat-00001-sources.jar: org/bson/internal/CodecCache.java <==
/*
 * Copyright 2008-present MongoDB, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.bson.internal;

import org.bson.codecs.Codec;
import org.bson.codecs.configuration.CodecConfigurationException;

import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import static java.lang.String.format;

final class CodecCache {
    private final ConcurrentMap<Class<?>, Optional<? extends Codec<?>>> codecCache =
            new ConcurrentHashMap<Class<?>, Optional<? extends Codec<?>>>();

    public boolean containsKey(final Class<?> clazz) {
        return codecCache.containsKey(clazz);
    }

    public void put(final Class<?> clazz, final Codec<?> codec){
        codecCache.put(clazz, Optional.of(codec));
    }

    @SuppressWarnings("unchecked")
    public synchronized <T> Codec<T> putIfMissing(final Class<T> clazz, final Codec<T> codec){
        if (codecCache.containsKey(clazz)) {
            return (Codec<T>) codecCache.get(clazz).get();
        } else {
            codecCache.put(clazz, Optional.of(codec));
            return codec;
        }
    }

    @SuppressWarnings("unchecked")
    public <T> Codec<T> getOrThrow(final Class<T> clazz) {
        if (codecCache.containsKey(clazz)) {
            Optional<? extends Codec<?>> optionalCodec = codecCache.get(clazz);
            if (!optionalCodec.isEmpty()) {
                return (Codec<T>) optionalCodec.get();
            }
        }
        throw new CodecConfigurationException(format("Can't find a codec for %s.", clazz));
    }
}

==> bson-4.3.2.redhat-00001-sources.jar: org/bson/internal/CodecCache.java <==
/*
 * Copyright 2008-present MongoDB, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.bson.internal;

import org.bson.codecs.Codec;
import org.bson.codecs.configuration.CodecConfigurationException;

import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;

import static java.lang.String.format;

final class CodecCache {
    private final ConcurrentMap<Class<?>, Optional<Codec<?>>> codecCache = new ConcurrentHashMap<>();

    public boolean containsKey(final Class<?> clazz) {
        return codecCache.containsKey(clazz);
    }

    public void put(final Class<?> clazz, final Codec<?> codec){
        codecCache.put(clazz, Optional.ofNullable(codec));
    }

    @SuppressWarnings("unchecked")
    public synchronized <T> Codec<T> putIfMissing(final Class<T> clazz, final Codec<T> codec) {
        Optional<Codec<?>> cachedCodec = codecCache.computeIfAbsent(clazz, clz -> Optional.of(codec));
        if (cachedCodec.isPresent()) {
            return (Codec<T>) cachedCodec.get();
        }
        codecCache.put(clazz, Optional.of(codec));
        return codec;
    }

    @SuppressWarnings("unchecked")
    public <T> Codec<T> getOrThrow(final Class<T> clazz) {
        return (Codec<T>) codecCache.getOrDefault(clazz, Optional.empty()).orElseThrow(
                () -> new CodecConfigurationException(format("Can't find a codec for %s.", clazz)));
    }
}
